/**
 * 
 */
package de.urszeidler.eclipse.callchain.acceleogenerators;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.acceleo.common.IAcceleoConstants;
import org.eclipse.acceleo.engine.service.AbstractAcceleoGenerator;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.BasicMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.Monitor;
import org.eclipse.emf.common.util.URI;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.executables.AbstractExecuteable;

/**
 * @author urs
 * 
 */
public class ModuleGenerator extends AbstractExecuteable implements GeneratorExecutable {

	private static final String ACCELEO_TEMPLATE = "acceleo.template";
	private AcceletoGenerator acceletoGenerator;

	private class AcceleoGeneratorConfig {
		private String moduleName;
		private String[] templateNames;
		private URI inputModel;
		private File outputFolder;
	}

	/**
	 * @author urs
	 * 
	 */
	public class AcceletoGenerator extends AbstractAcceleoGenerator {
		private String moduleName;
		private String[] templateNames;

		@Override
		public String getModuleName() {
			return moduleName;
		}

		@Override
		public String[] getTemplateNames() {
			return templateNames;
		}

		public void setModuleName(String moduleName) {
			this.moduleName = moduleName;
		}

		public void setTemplateNames(String[] templateNames) {
			this.templateNames = templateNames;
		}

		@Override
		protected URL findModuleURL(String moduleName) {
			try {
				URL moduleURL = ModuleGenerator.this.findModuleURL(moduleName);
				return moduleURL;
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();

			}
			return null;
		}

	}

	/**
	 * 
	 */
	public ModuleGenerator() {
	}

	@Override
	protected String validateGG(Generic_Generator generator) {
		try {
			AcceleoGeneratorConfig config = createAcceleoConfig(generator);

			if (config.moduleName == null)
				return "no module found";
			if (config.inputModel == null)
				return "no input model defined";
			if (config.outputFolder == null)
				return "no output Folder defined";
			if (config.templateNames == null)
				return "no template names defined";

		} catch (Exception e) {
			return e.getMessage();
		}
		return "";
	}

	@Override
	public void configure(Object generator) {
		super.configure(generator);
		doConfigure(this.generator);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	// @Override
	private void doConfigure(Generic_Generator generator) {
		try {
			AcceleoGeneratorConfig acceleoConfig = createAcceleoConfig(generator);

			List<?> arguments = Collections.EMPTY_LIST;
			acceletoGenerator = new AcceletoGenerator();
			acceletoGenerator.setModuleName(acceleoConfig.moduleName);
			acceletoGenerator.setTemplateNames(acceleoConfig.templateNames);

			acceletoGenerator.initialize(acceleoConfig.inputModel, acceleoConfig.outputFolder, arguments);

		} catch (IOException e) {
			e.printStackTrace();
			// TODO : we need to check for the exception here an in other places
		}

	}

	/**
	 * Stores all config data in the wrapper.
	 * 
	 * @param generator
	 * @return
	 */
	private AcceleoGeneratorConfig createAcceleoConfig(Generic_Generator generator) {
		AcceleoGeneratorConfig config2 = new AcceleoGeneratorConfig();
		config2.moduleName = findModuleName(generator);
		config2.templateNames = findTemplateNames(generator.getUses());
		config2.inputModel = findModel(generator);
		config2.outputFolder = findOutputFolder(generator);
		return config2;
	}

	/**
	 * Returns the target folder for the acceleo generator.
	 * 
	 * @param generator
	 * @return
	 */
	private File findOutputFolder(Generic_Generator generator) {
		File folder;
		EList<Artifact> produce = generator.getProduce();
		Artifact storeUri = getFirstArtifact(produce);

		folder = artifactToFileWorkspaceOrAbsolute(storeUri);
		return folder;
	}

	/**
	 * Creates an uri for the model, readable by the acceleo generator.
	 * 
	 * @param generator
	 * @return
	 */
	private URI findModel(Generic_Generator generator) {
		List<Model> allModels = getAllModels(generator.getUses());
		Model artifact = allModels.get(0);
		URI modelURI = artifactToUri(artifact);
		modelURI = URI.createURI(URI.decode(modelURI.toString()));
		return modelURI;
	}

	/**
	 * returns the Module name for the generator.
	 * 
	 * @param generator
	 * @return
	 */
	private String findModuleName(Generic_Generator generator) {
		Artifact module = findMainModule(generator.getUses());
		IFile artifactToIFile = artifactToIFile(module);
		String name = artifactToIFile.getName();
		int extensionLength = artifactToIFile.getFileExtension().length()+1;
		String moduleName = name.substring(0, name.length()- extensionLength);
		return moduleName;
	}

	/**
	 * Finds all template names from the list of Artfacts. By looking for the
	 * uri ACCELEO_TEMPLATE.
	 * 
	 * @param uses
	 * @return
	 */
	private String[] findTemplateNames(EList<Artifact> uses) {
		List<String> templateList = new ArrayList<String>();
		for (Artifact artifact : uses) {
			if (ACCELEO_TEMPLATE.equals(artifact.getUri()))
				templateList.add(artifact.getName());
		}

		String[] list = new String[] {};
		return templateList.toArray(list);
	}

	private Artifact findMainModule(EList<Artifact> uses) {
		Artifact artifact = findArtifactBySuffixPattern(IAcceleoConstants.EMTL_FILE_EXTENSION, uses);
		return artifact;

	}

	// TODO : check this out
	/**
	 * Tries to get the file from the workspace, if it's not exist it will
	 * interpreted as absolute uri.
	 * 
	 * @param artifact
	 * @return
	 */
	protected File artifactToFileWorkspaceOrAbsolute(Artifact artifact) {
		IContainer artifactToIContainter = artifactToIContainter(artifact);
		if (artifactToIContainter.exists()) {
			return artifactToIContainter.getLocation().toFile();
		}
		Path path = new Path(artifact.getUri());
		return path.makeAbsolute().toFile();

	}

	/**
	 * Here we resolve the module for the generator.
	 * 
	 * @param moduleName
	 * @return
	 * @throws MalformedURLException
	 */
	protected URL findModuleURL(String moduleName) throws MalformedURLException {
		if (!moduleName.endsWith('.' + IAcceleoConstants.EMTL_FILE_EXTENSION)) {
			moduleName += '.' + IAcceleoConstants.EMTL_FILE_EXTENSION;
		}

		Artifact artifact = findArtifactBySuffixPattern(moduleName);
		File file = artifactToAbsoluteFile(artifact);
		return file.toURI().toURL();

	}

	// private Artifact findArtifactByName(String moduleName) {
	// List<Artifact> allArtifacts = getAllArtifacts(generator.getUses());
	// return findArtifactByName(moduleName, allArtifacts);
	// }

	// /**
	// *
	// * @param moduleName
	// * @param allArtifacts
	// * @return
	// */
	// private Artifact findArtifactByName(String moduleName, List<Artifact>
	// allArtifacts) {
	// for (Artifact artifact : allArtifacts) {
	// if (artifact.getName().equals(moduleName))
	// return artifact;
	// }
	// return null;
	// }

	// private File artifactToAbsoluteFile(Artifact artifact) {
	// String uri = artifact.getUri();
	// File file = new File(uri);
	//
	// return file;
	// }

	/**
	 * Finds the first artifact having the moduleName as suffix.
	 * 
	 * @param moduleName
	 * @return
	 */
	private Artifact findArtifactBySuffixPattern(String moduleName) {
		List<Artifact> allArtifacts = getAllArtifacts(generator.getUses());
		return findArtifactBySuffixPattern(moduleName, allArtifacts);
	}

	/**
	 * Returns the first artifact witch uri matches the suffix pattern, or null.
	 * 
	 * @param moduleName
	 * @param allArtifacts
	 * @return
	 */
	private static Artifact findArtifactBySuffixPattern(String moduleName, List<Artifact> allArtifacts) {
		for (Artifact artifact : allArtifacts) {
			if (artifact.getUri().endsWith(moduleName))
				return artifact;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	//@Override
	public void execute(IProgressMonitor monitor) throws CoreException {
		Monitor m = BasicMonitor.toMonitor(monitor);
		try {
			acceletoGenerator.doGenerate(m);
		} catch (IOException e) {
			throw Activator.createCoreException(e);
		}
	}

}
