/**
 * 
 */
package de.urszeidler.eclipse.callchain.diagram.outline;

import java.util.Collection;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecoretools.diagram.ui.outline.AbstractModelNavigator;
import org.eclipse.emf.ecoretools.diagram.ui.outline.IOutlineMenuConstants;
import org.eclipse.emf.ecoretools.diagram.ui.outline.actions.CreateDiagramAction;
import org.eclipse.emf.edit.ui.action.CopyAction;
import org.eclipse.emf.edit.ui.action.CreateChildAction;
import org.eclipse.emf.edit.ui.action.CreateSiblingAction;
import org.eclipse.emf.edit.ui.action.CutAction;
import org.eclipse.emf.edit.ui.action.DeleteAction;
import org.eclipse.emf.edit.ui.action.PasteAction;
import org.eclipse.emf.edit.ui.action.RedoAction;
import org.eclipse.emf.edit.ui.action.UndoAction;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramWorkbenchPart;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.IPageSite;

import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.diagram.edit.parts.CallsEditPart;
import de.urszeidler.eclipse.callchain.diagram.part.CallchainDiagramEditorPlugin;

/**
 * @author urs
 * 
 */
public class CallchainNavigator extends AbstractModelNavigator {

	public CallchainNavigator(Composite parent, IDiagramWorkbenchPart diagEditor, IPageSite pageSite) {
		super(parent, diagEditor, pageSite);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.emf.ecoretools.diagram.ui.outline.AbstractModelNavigator#
	 * getAdapterFactory()
	 */
	@Override
	protected AdapterFactory getAdapterFactory() {
		return CallchainDiagramEditorPlugin.getInstance().getItemProvidersAdapterFactory();
	}

	/**
	 * @see org.eclipse.emf.ecoretools.diagram.ui.outline.AbstractModelNavigator#createDiagramsMenu(org.eclipse.jface.action.IMenuManager,
	 *      org.eclipse.emf.ecore.EObject)
	 */
	@Override
	protected void createDiagramsMenu(IMenuManager manager, EObject selectedObject) {
		super.createDiagramsMenu(manager, selectedObject);

		if (selectedObject instanceof Calls) {
			// final EObject selectedObj = selectedObject;
			manager.appendToGroup(IOutlineMenuConstants.NEW_GROUP,
					new CreateDiagramAction(selectedObject, getDiagramResource(), CallsEditPart.MODEL_ID,
							CallchainDiagramEditorPlugin.DIAGRAM_PREFERENCES_HINT));
		}
	}

	protected void initDragAndDrop() {
		super.initDragAndDrop();
	}

	@Override
	protected boolean isEMFMenuEnabledFor(EObject selectedObject) {

		return true;
	}

	protected void createEMFMenu(final IMenuManager manager, EObject selectedObject) {

		if (!isEMFMenuEnabledFor(selectedObject)) {

			return;
		}
		ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();

		final TransactionalEditingDomain editingDomain = TransactionUtil.getEditingDomain(getDiagramResource());
		final IStructuredSelection structuredSelection = new StructuredSelection(selectedObject);

		Collection<?> newChildDescriptors = editingDomain.getNewChildDescriptors(selectedObject, null);
		Collection<?> newSiblingDescriptors = editingDomain.getNewChildDescriptors(null, selectedObject);

		if (newChildDescriptors.size() > 0) {
			MenuManager menuManager = new MenuManager("childs");
			for (Object descriptor : newChildDescriptors) {
				menuManager.add(new CreateChildAction(getEditor(), structuredSelection, descriptor));
			}
			manager.add(menuManager);
		}
		if (newSiblingDescriptors.size() > 0) {
			MenuManager menuManager = new MenuManager("silbings");
			for (Object descriptor : newSiblingDescriptors) {
				menuManager.add(new CreateSiblingAction(getEditor(), structuredSelection, descriptor));
			}
			manager.add(menuManager);
		}
		manager.add(new Separator());

		UndoAction undoAction = new UndoAction(editingDomain);
		undoAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_UNDO));
		manager.add(new ActionContributionItem(undoAction));

		RedoAction redoAction = new RedoAction(editingDomain);
		redoAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_REDO));
		manager.add(new ActionContributionItem(redoAction));

		manager.add(new Separator());

		CutAction cutAction = new CutAction(editingDomain);
		cutAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_CUT));
		manager.add(new ActionContributionItem(cutAction));

		CopyAction copyAction = new CopyAction(editingDomain);
		copyAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
		manager.add(new ActionContributionItem(copyAction));

		PasteAction pasteAction = new PasteAction(editingDomain);
		pasteAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));
		manager.add(new ActionContributionItem(pasteAction));

		manager.add(new Separator());

		DeleteAction deleteAction = new DeleteAction(editingDomain);
		deleteAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		manager.add(new ActionContributionItem(deleteAction));

		deleteAction.updateSelection(structuredSelection);
		cutAction.updateSelection(structuredSelection);
		copyAction.updateSelection(structuredSelection);
		pasteAction.updateSelection(structuredSelection);

	}

}
