/**
 * 
 */
package de.urszeidler.eclipse.callchain.diagram.part;

import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.Request;
import org.eclipse.gmf.runtime.diagram.core.edithelpers.CreateElementRequestAdapter;
import org.eclipse.gmf.runtime.diagram.core.preferences.PreferencesHint;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateUnspecifiedTypeRequest;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewAndElementRequest;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewAndElementRequest.ViewAndElementDescriptor;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest.ViewDescriptor;
import org.eclipse.gmf.runtime.diagram.ui.tools.UnspecifiedTypeCreationTool;
import org.eclipse.gmf.runtime.diagram.ui.util.INotationType;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.IHintedType;
import org.eclipse.gmf.runtime.emf.type.core.requests.CreateElementRequest;
import org.eclipse.gmf.runtime.notation.Node;

import de.urszeidler.eclipse.callchain.generatorservice.GeneratorDescriptor;

@SuppressWarnings("unchecked")
public class GenericGeneratorCreationTool extends UnspecifiedTypeCreationTool {
		private final class GeneratorCreateRequest extends CreateUnspecifiedTypeRequest {
		private GeneratorCreateRequest(List elementTypes, PreferencesHint preferencesHint) {
			super(elementTypes, preferencesHint);
		}

		/**
		 * Creates a <code>CreateViewRequest</code> or
		 * <code>CreateViewAndElementRequest</code> for each creation hint and
		 * adds it to the map of requests.
		 */
		protected void createRequests() {
			for (Iterator iter = getElementTypes().iterator(); iter.hasNext();) {
				IElementType elementType = (IElementType) iter.next();
				
				getExtendedData().put("generator_descriptor", descriptor);
				
				Request request = null;
				if (elementType instanceof INotationType) {
					ViewDescriptor viewDescriptor = new ViewDescriptor(null,
						Node.class, ((INotationType) elementType)
							.getSemanticHint(), getPreferencesHint());
					request = new CreateViewRequest(viewDescriptor);
				} else if (elementType instanceof IHintedType) {
					ViewAndElementDescriptor viewDescriptor = new ViewAndElementDescriptor(
						new CreateElementRequestAdapter(new CreateElementRequest(
							elementType)), Node.class,
						((IHintedType) elementType).getSemanticHint(), getPreferencesHint());
					request = new CreateViewAndElementRequest(
							viewDescriptor);

					request.setExtendedData(getExtendedData());
				} else {
					ViewAndElementDescriptor viewDescriptor = new ViewAndElementDescriptor(
						new CreateElementRequestAdapter(new CreateElementRequest(
							elementType)), Node.class, getPreferencesHint());
					request = new CreateViewAndElementRequest(viewDescriptor);
					request.setExtendedData(getExtendedData());
				}

				request.setType(getType());
				requests.put(elementType, request);
			}
		}
	}

		private GeneratorDescriptor descriptor;
		private List elementTypes;

		public GenericGeneratorCreationTool(List elementTypes,GeneratorDescriptor descriptor) {
			super(elementTypes);
			this.descriptor = descriptor;
			this.elementTypes = elementTypes;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.gef.tools.TargetingTool#createTargetRequest()
		 */
		protected Request createTargetRequest() {
			return new GeneratorCreateRequest(elementTypes, getPreferencesHint());
		}
	}