/**
 * 
 */
package de.urszeidler.eclipse.callchain.eef;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

/**
 * @author urs
 * 
 */
public class NewEObjectWizard extends Wizard {

	/**
	 * @param eReference
	 * @param adapterFactory
	 * @param editingDomain
	 * @param instanciableTypesInHierarchy
	 * @param eObject
	 */
	public NewEObjectWizard(EReference eReference, AdapterFactory adapterFactory, EditingDomain editingDomain,
			List<EClass> instanciableTypesInHierarchy, EObject eObject) {
		super();
		this.eReference = eReference;
		this.adapterFactory = adapterFactory;
		this.editingDomain = editingDomain;
		this.instanciableTypesInHierarchy = instanciableTypesInHierarchy;
		this.eObject = eObject;
	}

	protected EReference eReference;
	protected AdapterFactory adapterFactory;
	protected EditingDomain editingDomain;
	protected List<EClass> instanciableTypesInHierarchy;
	protected EObject eObject;
	protected EObject newEObject;

	private class SelectTypeToCreatePage extends WizardPage {

		protected SelectTypeToCreatePage(String pageName, String title, ImageDescriptor titleImage) {
			super(pageName, title, titleImage);
		}

		private List<Button> buttons = new ArrayList<Button>();

		protected SelectTypeToCreatePage() {
			super("Create Type");
			this.setTitle("");
			this.setDescription("");
		}

		public void createControl(Composite parent) {
			Composite control = new Composite(parent, SWT.NONE);
			GridData gd = new GridData(GridData.FILL_BOTH);
			control.setLayoutData(gd);
			GridLayout layout = new GridLayout();
			control.setLayout(layout);

			for (final EClass eClass : instanciableTypesInHierarchy) {
				Button button = new Button(control, SWT.RADIO);
				button.setText(eClass.getName());
				button.addSelectionListener(new SelectionAdapter() {

					public void widgetSelected(SelectionEvent e) {
						newEObject = EcoreUtil.create(eClass);
						// mainPage.setInput(eObject);
					}
				});
				buttons.add(button);
			}
			if (buttons.size() > 0) {
				buttons.get(0).setSelection(true);
			} else {
				Label errorLabel = new Label(control, SWT.NONE);
				errorLabel.setText("Error non instanciable type found");
			}
			setControl(control);
		}

		@Override
		public boolean canFlipToNextPage() {
			return newEObject != null;
		}
	}

	@Override
	public void addPages() {
		if (!instanciableTypesInHierarchy.isEmpty()) {
			SelectTypeToCreatePage selectTypeToCreatePage = new SelectTypeToCreatePage();
			addPage(selectTypeToCreatePage);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		if (newEObject != null) {
			eObject.eSet(eReference, newEObject);
			return true;
		}

		return false;
	}

}
