/**
 * 
 */
package de.urszeidler.eclipse.callchain.eefgenerator;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.eef.EEFGen.EEFGenModel;
import org.eclipse.emf.eef.codegen.EEFCodegenPlugin;
import org.eclipse.emf.eef.codegen.extended.launcher.GenerateAll;
import org.osgi.framework.Bundle;

import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.editor.preferences.UriFieldEditor;
import de.urszeidler.eclipse.callchain.eefgenerator.preferences.PreferenceConstants;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.executables.AbstractExecuteable;

/**
 * @author urs
 * 
 */
public class EditorGenerator extends AbstractExecuteable implements GeneratorExecutable {
	public static final String JDT_CORE_SYMBOLIC_NAME = "org.eclipse.jdt.core"; //$NON-NLS-1$
	private static final String[] MM_URIS = Activator.getDefault().getPreferenceStore()
			.getString(PreferenceConstants.EEF_GEN_URIS).split(UriFieldEditor.LIST_SPILTT);// {GENMODELURI,GENMODELURI2008,GENMODELURI2009};{"http://www.eclipse.org/emf/eef/generation/1.0.0"};
	private List<EEFGenModel> eefGenModels;

	private class EEFGenerator {

		public void run(IProgressMonitor monitor) {
			try {
				if (eefGenModels != null) {
					for (final EEFGenModel eefGenModel : eefGenModels) {
						EcoreUtil.resolveAll(eefGenModel);
						final IContainer target = getGenContainer(eefGenModel);
						if (target != null) {
							int count = 2;
							if (eefGenModel.getEditionContexts() != null)
								count += eefGenModel.getEditionContexts().size() * 11;
							if (eefGenModel.getViewsRepositories() != null)
								count += eefGenModel.getViewsRepositories().size() * 5;
							monitor.beginTask("Generating EEF Architecture", count);
							final GenerateAll generator = new GenerateAll();
							generator.doGenerate(eefGenModel, target, monitor);
//							for (Iterator<IContainer> iterator = generator.getGenerationTargets().iterator(); iterator
//									.hasNext();) {
//								IContainer nextContainer = iterator.next();
//								nextContainer.refreshLocal(IResource.DEPTH_INFINITE, monitor);
//							}
							monitor.worked(1);
//							try {
//								if (isBundleLoaded(JDT_CORE_SYMBOLIC_NAME)) {
//									monitor.beginTask("Organize imports", 1);
//									Display.getDefault().asyncExec(new Runnable() {
//										public void run() {
//											IWorkbench wb = PlatformUI.getWorkbench();
//											IWorkbenchWindow win = wb.getActiveWorkbenchWindow();
//											IWorkbenchPage page = win.getActivePage();
//
//											IWorkbenchSite site = page.getActivePart().getSite();
//											ImportOrganizer.organizeImports(site, generator.getGenerationTargets());
//										}
//									});
//								}
//
//							} catch (Exception e) {
//								EEFCodegenPlugin.getDefault().logError(e);
//							}
						}
					}
				}
			} catch (IOException e) {
				EEFCodegenPlugin.getDefault().logError(e);
//			} catch (CoreException e) {
//				EEFCodegenPlugin.getDefault().logError(e);
			} finally {
				monitor.done();
			}
		}

		/**
		 * method defining if a bundle is loaded or not
		 * 
		 * @param name
		 *            the searched bundle
		 * @return <code>true</code> when the bundle is loaded
		 */
		public boolean isBundleLoaded(String name) {
			Bundle bundle = Platform.getBundle(name);
			return bundle != null && bundle.getState() == Bundle.ACTIVE;
		}

		/**
		 * Returns the container that the EEFGenModel use as generation
		 * directory
		 * 
		 * @param eefGenModel
		 *            the eefGenModel
		 * @return the generation directory
		 * @throws IOException
		 *             an error occurred during container creation
		 */
		public IContainer getGenContainer(EEFGenModel eefGenModel) throws IOException {
			if (eefGenModel != null) {
				if (eefGenModel.getGenDirectory() != null) {
					final IContainer target = (IContainer) ResourcesPlugin.getWorkspace().getRoot()
							.getFolder(new Path(eefGenModel.getGenDirectory()));
					return target;
				}
			}
			return null;
		}

	}

	/**
	 * 
	 */
	public EditorGenerator() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	// @Override
	public void configure(Object generator) {
		super.configure(generator);
		String string = validateGG((Generic_Generator) generator);
		if (!"".equals(string))
			return;

		// List<AbstractPropertiesGeneratorLauncher> list =
		// PropertiesGeneratorLaunchersServices.getInstance()
		// .getlaunchers();
		// propertiesGeneratorLauncher = list.get(0);
	}

	@Override
	protected String validateGG(Generic_Generator generator) {
		// Model model = findModelByMetaModel(generator.getUses(), MM_URIS);
		List<Model> findAllModels = findAllModels(generator.getUses(), MM_URIS);
		if (findAllModels.size() == 0)
			return "no EEFGenModels";

		eefGenModels = new ArrayList<EEFGenModel>();
		for (Model model : findAllModels) {

			Object loadEObject = loadEObject(model);
			if (loadEObject instanceof EEFGenModel) {
				EEFGenModel eefGenModel = (EEFGenModel) loadEObject;
				EcoreUtil.resolveAll(eefGenModel);
				eefGenModels.add(eefGenModel);
			}
		}

		return super.validateGG(generator);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	// @Override
	public void execute(IProgressMonitor monitor) {
		EEFGenerator eefGenerator = new EEFGenerator();
		eefGenerator.run(monitor);

	}

}
