/**
 * 
 */
package de.urszeidler.eclipse.callchain.eefgenerator;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.eef.codegen.core.initializer.AbstractPropertiesInitializer;
import org.eclipse.emf.eef.codegen.extended.initializer.EEFEditorModelInitializer;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;
import de.urszeidler.eclipse.callchain.generatorservice.Generatorservice;
import de.urszeidler.eclipse.callchain.generatorservice.executables.AbstractExecuteable;

/**
 * @author urs
 *
 */
public class EditorInitalizer extends AbstractExecuteable  implements GeneratorExecutable {

	private static final String EMF_GENMODEL = "http://www.eclipse.org/emf/2002/GenModel";
	private AbstractPropertiesInitializer propertiesInitializer;
	private URI modelURI;
	private IContainer targetFolder;
	//private EEFEditorModelInitializer propertiesInitializer;
	/**
	 * 
	 */
	public EditorInitalizer() {
	}

	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	//@Override
	public void configure(Object generator) {
		super.configure(generator);
		propertiesInitializer = null;
		String validate = validate(generator);
		if(!"".equals(validate))
			return;
		
		propertiesInitializer = new EEFEditorModelInitializer();
	}
	
	@Override
	protected String validateGG(Generic_Generator generator) {
		Model model = findModelByMetaModel(generator.getUses(), EMF_GENMODEL);
		if(model==null)
			return "no input model conformfs to http://www.eclipse.org/emf/2002/GenModel";
		
		modelURI = artifactToUri(model);
		if(modelURI==null)
			return "the model don't have a correct uri";
		
		EList<Artifact> produce = generator.getProduce();
		if(produce.size()==0)
			return "no output folder specified";
			
		Artifact artifact = produce.get(0);
		targetFolder = artifactToIContainter(artifact);
		if(targetFolder==null)
			return "target folder could not be found";
		
		return "";
	}
	/* (non-Javadoc)
	 * @see de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	//@Override
	public void execute(IProgressMonitor monitor) {
		if(propertiesInitializer==null)
			return;
		
		try {
			propertiesInitializer.initialize(modelURI, targetFolder);
		} catch (Exception e) {
			Generatorservice.logError("Error while initalize the editor model", e);
		}
	}


}
