/**
 * 
 */
package de.urszeidler.eclipse.callchain.emfgenerators.executables;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.codegen.ecore.CodeGenEcorePlugin;
import org.eclipse.emf.codegen.ecore.generator.Generator;
import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.codegen.ecore.genmodel.generator.GenBaseGeneratorAdapter;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.BasicMonitor;
import org.eclipse.emf.common.util.Diagnostic;

import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.emfgenerators.Activator;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;

/**
 * @author urs
 * 
 */
public class EmfCodegeneration extends AbstractEmfExecutable implements GeneratorExecutable {
	protected static class ProjectType {
		protected Object type;
		protected String name;

		public ProjectType(Object type, String name) {
			this.type = type;
			this.name = name;
		}

		public Object getType() {
			return type;
		}

		public String getName() {
			return name;
		}
	}

	private ProjectType[] projectTypes;
	private Generator code_generator;

	/**
	 * 
	 */
	public EmfCodegeneration() {
		code_generator = new Generator();
		this.projectTypes = new ProjectType[] {
				new ProjectType(GenBaseGeneratorAdapter.MODEL_PROJECT_TYPE, CodeGenEcorePlugin.INSTANCE
						.getString("_UI_ModelProject_name")),
				new ProjectType(GenBaseGeneratorAdapter.EDIT_PROJECT_TYPE, CodeGenEcorePlugin.INSTANCE
						.getString("_UI_EditProject_name")),
				new ProjectType(GenBaseGeneratorAdapter.EDITOR_PROJECT_TYPE, CodeGenEcorePlugin.INSTANCE
						.getString("_UI_EditorProject_name")),

				new ProjectType(GenBaseGeneratorAdapter.TESTS_PROJECT_TYPE, CodeGenEcorePlugin.INSTANCE
						.getString("_UI_TestsProject_name")) };
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * configure(de.urszeidler.eclipse.callchain.Generic_Generator)
	 */
	@Override
	public void configure(Object generator) {
		super.configure(generator);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable#
	 * execute(org.eclipse.core.runtime.IProgressMonitor)
	 */
	//@Override
	public void execute(IProgressMonitor progressMonitor) throws CoreException {
		GenModel genModel = (GenModel) getGenModel();
		try {
			progressMonitor.beginTask("", projectTypes.length);
			try {
				genModel.setCanGenerate(true);
				code_generator.setInput(genModel);
				BasicDiagnostic diagnostic = new BasicDiagnostic("", 0, "", null);

				for (int i = 0; i < projectTypes.length; i++) {
					diagnostic.add(code_generator.generate(genModel, projectTypes[i].getType(), projectTypes[i]
							.getName(), BasicMonitor.toMonitor(new SubProgressMonitor(progressMonitor, 1))));

					if (!canContinue(diagnostic)) {
						break;
					}
				}

				if (diagnostic.getSeverity() != Diagnostic.OK) {
				}
			} catch (Exception e) {
				throw Activator.createCoreException(e);
			}

		} finally {
			progressMonitor.done();
		}
	}

	@Override
	protected String validateGG(Generic_Generator generator) {
		Model model = findModelByMetaModel(generator.getUses(), GENNODEL_URI);
		if (model == null)
			return "No model conforms to : "+GENNODEL_URI+" Metamodel.";

		return "";
	}

	private Object getGenModel() {
		Model model = findModelByMetaModel(generator.getUses(), GENNODEL_URI);
		return loadEObject(model);
	}

	// Stop only on cancel.
	//
	protected boolean canContinue(Diagnostic diagnostic) {
		return diagnostic.getSeverity() != Diagnostic.CANCEL;
	}

}
