/**
 * 
 */
package de.urszeidler.eclipse.callchain.emfgenerators.executables;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.BasicMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.Monitor;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.importer.ModelImporter;
import org.eclipse.emf.importer.ecore.EcoreImporter;

import de.urszeidler.eclipse.callchain.Artifact;
import de.urszeidler.eclipse.callchain.Generic_Generator;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.emfgenerators.Activator;
import de.urszeidler.eclipse.callchain.generatorservice.GeneratorExecutable;

/**
 * @author urs
 * 
 */
public class EmfGenmodelImporterExecutable extends AbstractEmfExecutable implements GeneratorExecutable {
	private static final String ECORE = "#EMF";
	private static final String XSD_EXTENSION = "xsd";
	private static final int ECORE_TYPE = 1;
	private static final int XSD_TYPE = 2;
	private ModelImporter importer;

	/**
	 * 
	 */
	public EmfGenmodelImporterExecutable() {
	}

	@Override
	public void configure(Object gen) {
		super.configure(gen);

		int type = getImporterType(generator.getUses());
		switch (type) {
		case ECORE_TYPE:
			configureEcoreImporter(generator);
			break;
		case XSD_TYPE:
			configureXSDImporter(generator);
			break;

		default:
			break;
		}
	}

	private void configureXSDImporter(Generic_Generator generator) {
		importer = new EcoreImporter();
		Model targetGenModel = findModelByMetaModel(generator.getProduce(), GENNODEL_URI);
		if (targetGenModel == null)
			return;
		Artifact sourceEcoreModel = getArtifactsByExtension(generator.getUses(), XSD_EXTENSION);
		if (sourceEcoreModel == null)
			return;

		// String strUri = sourceEcoreModel.getUri();
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		IFile currentEcoreFile = wksroot.getFile(new Path(sourceEcoreModel.getUri()));
		IFile currentGenFile = wksroot.getFile(new Path(targetGenModel.getUri()));

		importer.setModelProjectName(currentEcoreFile.getProject().getName());
		importer.setModelFile(currentEcoreFile);
//		List<EPackage> epackage = collectEPackages(sourceEcoreModel);
//		importer.getEPackages().addAll(epackage);
//		for (EPackage package_ : epackage) {
//			importer.getEPackageConvertInfo(package_).setConvert(true);
//			importer.getEPackageImportInfo(package_).setEcoreFileName(currentEcoreFile.getName());
//		}

		importer.addGenModelToResource(true);
		importer.setGenModelContainerPath(currentGenFile.getParent().getFullPath());
		importer.setGenModelFileName(currentGenFile.getName());
	}

	private int getImporterType(EList<Artifact> artifacts) {
		Model model = findModelByMetaModel(artifacts, ECORE);
		if (model != null)
			return ECORE_TYPE;

		Artifact artifact = getArtifactsByExtension(artifacts, XSD_EXTENSION);
		if (artifact != null)
			return XSD_TYPE;

		return 0;
	}

	private Artifact getArtifactsByExtension(EList<Artifact> artifacts, String extension) {
		for (Artifact artifact : artifacts) {
			String uri = artifact.getUri();
			if (uri != null)
				if (uri.toLowerCase().endsWith(extension.toLowerCase()))
					return artifact;
		}
		return null;
	}
	
	@Override
	protected String validateGG(Generic_Generator generator) {
		Model model = findModelByMetaModel(generator.getProduce(), GENNODEL_URI);
		if (model == null)
			return "No produced model conforms to : "+GENNODEL_URI+" Metamodel.";
		
		model = findModelByMetaModel(generator.getUses(), ECORE);
		if (model == null)
			return "No used model conforms to : "+ECORE+" Metamodel.";
		
		return "";
	}

	private void configureEcoreImporter(Generic_Generator generator) {
		importer = new EcoreImporter();
		Model targetGenModel = findModelByMetaModel(generator.getProduce(), GENNODEL_URI);
		if (targetGenModel == null)
			return;
		Model sourceEcoreModel = findModelByMetaModel(generator.getUses(), ECORE);
		if (sourceEcoreModel == null)
			return;

		// String strUri = sourceEcoreModel.getUri();
		IWorkspace wks = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot wksroot = wks.getRoot();

		IFile currentEcoreFile = wksroot.getFile(new Path(sourceEcoreModel.getUri()));
		IFile currentGenFile = wksroot.getFile(new Path(targetGenModel.getUri()));

		importer.setModelProjectName(currentEcoreFile.getProject().getName());
		importer.setModelFile(currentEcoreFile);
		List<EPackage> epackage = collectEPackages(sourceEcoreModel);
		importer.getEPackages().addAll(epackage);
		for (EPackage package_ : epackage) {
			importer.getEPackageConvertInfo(package_).setConvert(true);
			importer.getEPackageImportInfo(package_).setEcoreFileName(currentEcoreFile.getName());
		}

		importer.addGenModelToResource(true);
		importer.setGenModelContainerPath(currentGenFile.getParent().getFullPath());
		importer.setGenModelFileName(currentGenFile.getName());
	}

	@SuppressWarnings("unchecked")
	private List<EPackage> collectEPackages(Model sourceEcoreModel) {
		Object loadEObject = loadEObject(sourceEcoreModel);
		if (loadEObject instanceof List) {
			return (List<EPackage>) loadEObject;

		} else {
			List<EPackage> list = new ArrayList<EPackage>();
			list.add((EPackage) loadEObject);
			return list;
		}
	}

	//@Override
	public void execute(IProgressMonitor monitor) throws CoreException {
		Monitor bmonitor = BasicMonitor.toMonitor(monitor);
		try {
			importer.prepareGenModelAndEPackages(bmonitor);
			importer.saveGenModelAndEPackages(bmonitor);
		} catch (Exception e) {

			e.printStackTrace();
			throw Activator.createCoreException(e);
		}

	}

}
