package de.urszeidler.eclipse.callchain.executors;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.m2m.atl.common.ATLExecutionException;
import org.eclipse.m2m.atl.common.ATLLaunchConstants;
import org.eclipse.m2m.atl.core.ATLCoreException;
import org.eclipse.m2m.atl.core.launch.ILauncher;
import org.eclipse.m2m.atl.core.service.CoreService;
import org.eclipse.m2m.atl.core.service.LauncherService;

import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;

/**
 * The method "launch" is launched when you click on the button "Run" or
 * "Debug".
 * 
 * @author <a href="mailto:william.piers@obeo.fr">William Piers</a>
 * @author <a href="mailto:freddy.allilaire@obeo.fr">Freddy Allilaire</a>
 */
public class CallchainAtlLaunchConfigurationDelegate {
	String DISABLEATTRIBUTEHELPERCACHE = "disableAttributeHelperCache";
	private static Map<String, IFile> moduleFilesByModuleName;

	/**
	 * {@inheritDoc}
	 * 
	 * @param clearConsole
	 * @param option_derived
	 * @param isRefining
	 * @param atlCompiler
	 * @param launcherName
	 * @param fileName
	 * 
	 * @see org.eclipse.debug.core.model.ILaunchConfigurationDelegate#launch(org.eclipse.debug.core.ILaunchConfiguration,
	 *      java.lang.String, org.eclipse.debug.core.ILaunch,
	 *      org.eclipse.core.runtime.IProgressMonitor)
	 */
	@SuppressWarnings("unchecked")
	public void launch(String mode, IProgressMonitor monitor, Object clearConsole, boolean option_derived,
			boolean isRefining, String atlCompiler, String launcherName, String fileName,
			Map<String, String> sourceModels, Map<String, String> targetModels, Map<String, String> modelPaths,
			Map<String, String> libs, List<String> superimps, Map<String, String> modelHandlers,
			Map<String, Object> options) throws CoreException {

		moduleFilesByModuleName = new HashMap<String, IFile>();
		if (monitor == null) {
			monitor = new NullProgressMonitor();
		}


		boolean isRefiningTraceMode = "atl2006".equals(atlCompiler) && isRefining; //$NON-NLS-1$
		options.put("isRefiningTraceMode", new Boolean(isRefiningTraceMode).toString()); //$NON-NLS-1$
		//		options.put("launch", launch); //$NON-NLS-1$
		options.put("monitor", monitor); //$NON-NLS-1$


		Map<String, String> launchConfigModelPaths = new HashMap<String, String>();// configuration.getAttribute(ATLLaunchConstants.PATH,
		// Collections.EMPTY_MAP);
		launchConfigModelPaths.putAll(sourceModels);
		launchConfigModelPaths.putAll(targetModels);
		 Map<String, String> modelPaths1 =
		 convertPaths(modelPaths);

		options.put(ATLLaunchConstants.OPTION_MODEL_HANDLER, modelHandlers);

		// String clearConsole = options.get(AdvancedTab.OPTION_CLEAR);

		ILauncher launcher = null;
		try {
			// API extensions managementC
			launcher = CoreService.getLauncher(launcherName);
			System.out.println(CoreService.getExtractorsNames()); 
			System.out.println(CoreService.getLaunchersNames().toString()); 
			System.out.println(CoreService.getLauncherOptions(launcherName)); 
		} catch (ATLCoreException e) {
			CallchainEditPlugin.log( e.getMessage(),IStatus.ERROR, e);
		}

		if (launcher == null) {
			String[] registeredLaunchers = CoreService.getLaunchersNames();
			CallchainEditPlugin.log("The Laucher was not found :"+launcherName+" registered Lauchers are ; "
					+ Arrays.toString(registeredLaunchers),IStatus.ERROR);
			
			return;
		}

		// ATL modules
		IFile currentAtlFile = ResourcesPlugin.getWorkspace().getRoot().getFile(Path.fromOSString(fileName));
		String currentExtension = currentAtlFile.getFileExtension().toLowerCase();
		if (currentExtension.equals("atl")) { //$NON-NLS-1$
			String currentAsmPath = currentAtlFile.getFullPath().toString().substring(0,
					currentAtlFile.getFullPath().toString().length() - currentExtension.length())
					+ "asm"; //$NON-NLS-1$
			currentAtlFile = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(currentAsmPath));
		}

		if (!addLaunchedModule(currentAtlFile)) {
			return;
		}

		InputStream asmInputStream = currentAtlFile.getContents();
		InputStream[] modules = new InputStream[superimps.size() + 1];
		modules[0] = asmInputStream;
		for (int i = 1; i < modules.length; i++) {
			String moduleFileName = superimps.get(i - 1);
			IFile moduleFile = ResourcesPlugin.getWorkspace().getRoot().getFile(Path.fromOSString(moduleFileName));
			if (!addLaunchedModule(moduleFile)) {
				return;
			}
			modules[i] = moduleFile.getContents();
		}

		// Libraries
		Map<String, InputStream> libraries = new HashMap<String, InputStream>();
		for (Iterator<String> i = libs.keySet().iterator(); i.hasNext();) {
			String libName = i.next();
			IFile libFile = ResourcesPlugin.getWorkspace().getRoot().getFile(Path.fromOSString(libs.get(libName)));
			if (!addLaunchedModule(libFile)) {
				return;
			}
			libraries.put(libName, libFile.getContents());
		}

		// check for cancellation
		if (monitor.isCanceled()) {
			return;
		}

		try {
			if (isRefiningTraceMode) {
				/*
				 * TODO: improve ATL header syntax to recognize inout models.
				 * Apply those changes to launch config. Current workaround:
				 * refined models list must match output models list to be
				 * saved, with respect to the declaration order.
				 */
				Iterator<String> sourceIterator = sourceModels.keySet().iterator();
				Iterator<String> targetIterator = targetModels.keySet().iterator();

				List<String> orderedInput = // configuration.getAttribute(ATLLaunchConstants.ORDERED_INPUT,
						Collections.EMPTY_LIST;//);
				if (!orderedInput.isEmpty()) {
					sourceIterator = orderedInput.iterator();
				}

				List<String> orderedOutput = //configuration.getAttribute(ATLLaunchConstants.ORDERED_OUTPUT,
						Collections.EMPTY_LIST;//);
				if (!orderedOutput.isEmpty()) {
					targetIterator = orderedOutput.iterator();
				}

				Map<String, String> newTargetModels = new HashMap<String, String>();
				newTargetModels.putAll(targetModels);
				List<String> targetToRemove = new ArrayList<String>();

				while (sourceIterator.hasNext()) {
					String sourceModelName = sourceIterator.next();
					String sourceMetamodelName = sourceModels.get(sourceModelName);

					// Lookup for a matching target model (same metamodel)
					while (targetIterator.hasNext()) {
						String targetModelName = targetIterator.next();
						String targetMetamodelName = targetModels.get(targetModelName);

						// Ignore previously used target models
						if (targetMetamodelName.equals(sourceMetamodelName)
								&& !targetToRemove.contains(targetModelName)) {
							String targetModelPath = modelPaths.get(targetModelName);

							// Compute the inout model path (for extraction)
							String refinedModelPathName = LauncherService.getRefinedModelName(sourceModelName);
							modelPaths.put(refinedModelPathName, targetModelPath);
							targetToRemove.add(targetModelName);
							break;
						}
					}
				}

				for (String key : targetToRemove) {
					newTargetModels.remove(key);
				}

				LauncherService.launch(mode, monitor, launcher, Collections.EMPTY_MAP, sourceModels, newTargetModels,
						modelPaths1, options, libraries, modules);
			} else {
				LauncherService.launch(mode, monitor, launcher, sourceModels, Collections.EMPTY_MAP, targetModels,
						modelPaths1, options, libraries, modules);
			}

			if (option_derived) { //$NON-NLS-1$
				// Set generated files as derived
				for (String targetModel : targetModels.keySet()) {
					String path = launchConfigModelPaths.get(targetModel);
					setDerived(path);
				}
			}

		} catch (ATLCoreException e) {
			CallchainEditPlugin.log( e.getMessage(),IStatus.ERROR, e);
			return;
		} catch (ATLExecutionException e) {
			CallchainEditPlugin.log( e.getMessage(),IStatus.ERROR, e);
		} finally {
			monitor.done();
		}
	}

	private static boolean addLaunchedModule(IFile file) {
		if (!file.exists()) {
			CallchainEditPlugin.log("Atl file don't exsitst "+file.getFullPath().toString(), IStatus.WARNING);
//			ATLLogger.severe(Messages.getString("AtlLaunchConfigurationDelegate.FILE_NOT_EXIST", file.getFullPath())); //$NON-NLS-1$
			return false;
		}
		IFile atlFile = file;
		if (atlFile != null) {
			String ext = atlFile.getFileExtension().toLowerCase();
			if (ext.equals("asm")) { //$NON-NLS-1$
				String path = atlFile.getFullPath().toString().substring(0,
						atlFile.getFullPath().toString().length() - ext.length())
						+ "atl"; //$NON-NLS-1$
				atlFile = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(path));
			}
			if (atlFile.isAccessible()) {
				moduleFilesByModuleName.put(computeModuleName(file), atlFile);
			}
		}
		return true;
	}

	/**
	 * Returns the file matching the given module name.
	 * 
	 * @param moduleName
	 *            the module name
	 * @return the file
	 */
	public static IFile getFileFromModuleName(String moduleName) {
		if (moduleFilesByModuleName != null) {
			return moduleFilesByModuleName.get(moduleName);
		}
		return null;
	}

	private static String computeModuleName(IFile file) {
		String res = file.getName();
		try {
			InputStreamReader streamReader = new InputStreamReader(file.getContents());
			BufferedReader buffer = new BufferedReader(streamReader);
			String line = ""; //$NON-NLS-1$
			while (null != (line = buffer.readLine())) {
				if (line.contains("<constant value=")) { //$NON-NLS-1$
					res = line.split("\\\"")[1]; //$NON-NLS-1$
					buffer.close();
					streamReader.close();
					break;
				}
			}
		} catch (Throwable e) {
			// DO NOTHING
		}
		return res;
	}

	/**
	 * Convert model map paths.
	 * 
	 * @param modelPaths
	 *            the model path map
	 * @return the converted map
	 */
	public static Map<String, String> convertPaths(Map<String, String> modelPaths) {
		Map<String, String> result = new HashMap<String, String>();
		for (Iterator<String> iterator = modelPaths.keySet().iterator(); iterator.hasNext();) {
			String modelName = iterator.next();
			String modelPath = modelPaths.get(modelName);
			result.put(modelName, convertPath(modelPath));
		}
		return result;
	}

	/**
	 * Convert "launch configuration style" paths to EMF uris:
	 * <ul>
	 * <li>ext:<i>path</i> => file:<i>path</i> (file system resource)</li>
	 * <li>uri:<i>uri</i> => <i>uri</i> (EMF uri)</li>
	 * <li><i>path</i> => platform:/resource/<i>path</i> (workspace resource)</li>
	 * </ul>
	 * Unchanged paths:
	 * <ul>
	 * <li>platform:/plugin/<i>path</i> (plugin resource)</li>
	 * <li>pathmap:<i>path</i> (pathmap resource, e.g. UML2 profile)</li>
	 * </ul>
	 * 
	 * @param path
	 *            the path as created by the launchConfiguration
	 * @return the converted path
	 */
	public static String convertPath(String path) {
		if (path.startsWith("ext:")) { //$NON-NLS-1$
			return path.replaceFirst("ext:", "file:/"); //$NON-NLS-1$ //$NON-NLS-2$
		} else if (path.startsWith("uri:")) { //$NON-NLS-1$
			return path.substring(4);
		} else if (path.startsWith("#") || path.startsWith("platform:") || path.startsWith("pathmap")) { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			return path;
		}
		return "platform:/resource" + path; //$NON-NLS-1$
	}

	private void setDerived(String filePath) {
		if (Platform.isRunning()) {
			try {
				IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(filePath));
				if (file.exists()) {
					file.setDerived(true);
				}
			} catch (IllegalStateException e) {
				CallchainEditPlugin.log( e.getMessage(),IStatus.ERROR, e);
			} catch (CoreException e) {
				CallchainEditPlugin.log( e.getMessage(),IStatus.ERROR, e);
			}
		}
	}

}
