/**
 * 
 */
package de.urszeidler.eclipse.callchain.generatorservice;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.themes.ColorUtil;

/**
 * @author urs
 * Is used as a descriptor and a simple factory.
 * 
 */
public class GeneratorDescriptor {
	private String id;
	private String name;
	private String smallIconPath;
	private String largeIconPath;
	private String color;
	private String className;
	private String constributor;
	private Class<? extends GeneratorExecutable> generatorClass;
	private Color genColor;
	private String description;
	private String category;
	
	/**
	 * the factory method
	 * @return
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public GeneratorExecutable createExecutable() throws InstantiationException, IllegalAccessException {
		if (generatorClass == null)
			throw new IllegalArgumentException("class may not be null");

		return generatorClass.newInstance();
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getColor() {
		return color;
	}

	public void setColor(String color) {
		this.color = color;
	}

	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

	public ImageDescriptor getSmallIcon() {
		if (smallIconPath == null)
			return null;

		return Generatorservice.getDefault().getBundledImageDescriptor(constributor, smallIconPath);
	}

	public ImageDescriptor getLargeIcon() {
		if (largeIconPath == null)
			return null;

		return Generatorservice.getDefault().getBundledImageDescriptor(constributor, largeIconPath);
	}

	public Image getSmallImage() {
		if (smallIconPath == null)
			return null;

		return Generatorservice.getDefault().getBundledImage(constributor, smallIconPath);
	}

	public Image getLargeImage() {
		if (largeIconPath == null)
			return null;

		return Generatorservice.getDefault().getBundledImage(constributor, largeIconPath);
	}

	public String getSmallIconPath() {
		return smallIconPath;
	}

	public void setSmallIconPath(String smallIconPath) {
		this.smallIconPath = smallIconPath;
	}

	public String getLargeIconPath() {
		return largeIconPath;
	}

	public void setLargeIconPath(String largeIconPath) {
		this.largeIconPath = largeIconPath;
	}

	public void setConstributor(String constributer) {
		this.constributor = constributer;

	}

	public String getConstributor() {
		return constributor;
	}

	public void setGeneratorClass(Class<? extends GeneratorExecutable> clazz) {
		this.generatorClass = clazz;

	}

	public Color getGeneratorColor() {
		if (genColor == null) {
			// ColorRegistry colorRegistry = new ColorRegistry();
			try {
				RGB colorValue = ColorUtil.getColorValue(color);
				genColor = new Color(null, colorValue);

			} catch (Exception e) {
				// TODO: handle exception
			}
		}

		return genColor;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getCategory() {
		return category;
	}

	public void setCategory(String category) {
		this.category = category;
	}
}
