/*
 * (c) urs zeidler
 */
package de.urszeidler.eclipse.callchain.importer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import de.urszeidler.eclipse.callchain.AtlLibary;
import de.urszeidler.eclipse.callchain.AtlRes;
import de.urszeidler.eclipse.callchain.Call;
import de.urszeidler.eclipse.callchain.CallchainFactory;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.MM;
import de.urszeidler.eclipse.callchain.MMhandler;
import de.urszeidler.eclipse.callchain.Model;
import de.urszeidler.eclipse.callchain.provider.CallchainEditPlugin;

public class ImportFromLauchConfig {

    public void saveCalls(URI filePath, Calls calls) {

        ResourceSetImpl impl = new ResourceSetImpl();

        org.eclipse.emf.ecore.resource.Resource resource = impl
                .createResource(filePath);

        resource.getContents().add(calls);
        try {
            resource.save(Collections.EMPTY_MAP);
        } catch (IOException e) {
            CallchainEditPlugin.log(e.getLocalizedMessage(), IStatus.ERROR);
        }

    }

    @SuppressWarnings( { "unused", "unchecked" })
    public Calls createCallchainByLauchConfig(IFile file) {
        Calls calls = null;
        ILaunchManager launchManager = DebugPlugin.getDefault()
                .getLaunchManager();

        ILaunchConfiguration launchConfiguration = launchManager
                .getLaunchConfiguration(file);

        try {
            String atlFilename = launchConfiguration.getAttribute(
                    AtlLauncherTools.ATLFILENAME, "");
            Map libList = launchConfiguration.getAttribute(
                    AtlLauncherTools.LIBS, new HashMap());
            Map mapPath = launchConfiguration.getAttribute(
                    AtlLauncherTools.PATH, new HashMap());

            Map mapModelIn = launchConfiguration.getAttribute(
                    AtlLauncherTools.INPUT, new HashMap());
            Map mapModelOut = launchConfiguration.getAttribute(
                    AtlLauncherTools.OUTPUT, new HashMap());
            Map mapModelType = launchConfiguration.getAttribute(
                    AtlLauncherTools.MODELTYPE, new HashMap());
            Map mapModelHandler = launchConfiguration.getAttribute(
                    AtlLauncherTools.MODELHANDLER, new HashMap());

            List superList = launchConfiguration.getAttribute(
                    AtlLauncherTools.SUPERIMPOSE, new ArrayList());

            calls = CallchainFactory.eINSTANCE.createCalls();
            Call call = CallchainFactory.eINSTANCE.createCall();
            call.setName(file.getName());
            call.setPrintExecutionTime(launchConfiguration.getAttribute(
                    AtlLauncherTools.OPTION_PRINT_EXECUTION_TIME, false));
            call.setShowSummary(launchConfiguration.getAttribute(
                    AtlLauncherTools.OPTION_SHOW_SUMMARY, false));
            call.setAllowInterModelReferences(launchConfiguration.getAttribute(
                    AtlLauncherTools.AllowInterModelReferences, false));
            call.setContinueAfterErrors(launchConfiguration.getAttribute(
                    AtlLauncherTools.OPTION_CONTINUE_AFTER_ERROR, false));
            call.setProfile(launchConfiguration.getAttribute(
                    AtlLauncherTools.OPTION_PROFILE, false));
            calls.getCall().add(call);

            AtlRes atlRes = CallchainFactory.eINSTANCE.createAtlRes();
            call.setRes(atlRes);
            atlRes.setName(new Path(atlFilename).lastSegment());
            atlRes.setUri(atlFilename);
            calls.getArtifacts().add(atlRes);

            for (Object en : libList.entrySet()) {
                Entry entry = (Entry) en;

                AtlLibary atlLibary = CallchainFactory.eINSTANCE
                        .createAtlLibary();
                atlLibary.setName((String) entry.getKey());
                atlLibary.setUri((String) entry.getValue());
                calls.getArtifacts().add(atlLibary);

            }

            for (Object object : superList) {
                AtlRes sAtlRes = CallchainFactory.eINSTANCE.createAtlRes();
                atlRes.getSuperimpose().add(sAtlRes);
                calls.getArtifacts().add(sAtlRes);
                String atlFile = (String) object;
                String currentAsmPath = atlFile.substring(0,
                        atlFile.length() - 3)
                        + "atl";

                sAtlRes.setUri(currentAsmPath);

                sAtlRes.setName(new Path((String) object).lastSegment());

            }

            for (Object en : mapPath.entrySet()) {
                if (en instanceof Entry) {
                    Entry entry = (Entry) en;

                    Object key = entry.getKey();
                    Object object = mapModelType.get(key);
                    if (object.equals("METAMODELOUTPUT")
                            || object.equals("METAMODELINPUT")) {
                        MM mm = CallchainFactory.eINSTANCE.createMM();
                        mm.setName((String) key);
                        mm.setUri((String) entry.getValue());
                        mm.setHandler(MMhandler.get((String) mapModelHandler
                                .get(key)));

                        calls.getArtifacts().add(mm);
                    }

                    if (object.equals("MODELINPUT")) {
                        Model model = CallchainFactory.eINSTANCE.createModel();
                        model.setName((String) key);
                        model.setUri((String) entry.getValue());
                        calls.getArtifacts().add(model);
                        call.getIn().add(model);
                    }

                    if (object.equals("MODELOUTPUT")) {
                        Model model = CallchainFactory.eINSTANCE.createModel();
                        model.setName((String) key);
                        model.setUri((String) entry.getValue());
                        calls.getArtifacts().add(model);
                        call.getOut().add(model);
                    }

                }// end of if (en instanceof Entry)
            }

        } catch (CoreException e) {
            CallchainEditPlugin.log(e.getLocalizedMessage(), IStatus.ERROR);
        }

        return calls;
    }

    private interface AtlLauncherTools {

        public static final String ATLVM = "ATL VM";
        public static final String ATLFILENAME = "ATL File Name";
        public static final String AllowInterModelReferences = "AllowInterModelReferences";
        public static final String PROJECTNAME = "Project Name";
        public static final String OUTPUT = "Output";
        public static final String INPUT = "Input";
        public static final String NULLPARAMETER = "<null>";
        public static final String PATH = "Path";
        public static final String HOST = "Host";
        public static final String PORT = "Port";
        public static final String LIBS = "Libs";

        public static final String MODEL_OUTPUT = "MODELOUTPUT";
        public static final String MODEL_INPUT = "MODELINPUT";
        public static final String METAMODEL_OUTPUT = "METAMODELOUTPUT";
        public static final String METAMODEL_INPUT = "METAMODELINPUT";

        public static final String SUPERIMPOSE = "Superimpose";
        public static final String MODELTYPE = "ModelType";
        public static final String INJECTOR = "Injector";
        public static final String EXTRACTOR = "Extractor";
        public static final String MODELHANDLER = "Model Handler";
        public static final String MODEDEBUG = "Mode Debug";
        public static final String PATHICONATL = "icons/atllogo_icon.gif";

        public static final String OPTION_CONTINUE_AFTER_ERROR = "continueAfterError";
        public static final String OPTION_DISABLE_ATTRIBUTE_HELPER_CACHE = "disableAttributeHelperCache";
        public static final String OPTION_ALLOW_TARGET_NAVIGATION = "allowTargetNavigation";
        public static final String OPTION_ALLOW_SOURCE_MODIFICATION = "allowSourceModification";
        public static final String OPTION_ALLOW_CONTAINER_REASSIGNMENT = "allowContainerReassignment";
        public static final String OPTION_PRINT_EXECUTION_TIME = "printExecutionTime";
        public static final String OPTION_STEP = "step";
        public static final String OPTION_SHOW_SUMMARY = "showSummary";
        public static final String OPTION_PROFILE = "profile";
        public static final String[] additionalParamIds = new String[] {
                // ALLOW_TARGET_NAVIGATION,
                // ALLOW_SOURCE_MODIFICATION,
                // ALLOW_CONTAINER_REASSIGNMENT,
                OPTION_CONTINUE_AFTER_ERROR,
                OPTION_DISABLE_ATTRIBUTE_HELPER_CACHE,
                OPTION_PRINT_EXECUTION_TIME, OPTION_STEP, OPTION_SHOW_SUMMARY,
                OPTION_PROFILE, };
    }
}
