/**
 * 
 */
package de.urszeidler.eclipse.callchain.lauching.ant;

import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.EList;

import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.Property;
import de.urszeidler.eclipse.callchain.executors.MCcallchain;
import de.urszeidler.eclipse.callchain.lauching.core.CallchainLaunchConfigurationDelegate;

/**
 * @author urs
 * 
 */
public class Atlfow extends Task {
	/**
	 * @author urs
	 *
	 */
	public class AntLogMonitor extends NullProgressMonitor {
		@Override
		public void beginTask(String name, int totalWork) {
			log("start :"+name);
		}

		@Override
		public void subTask(String name) {
			log(name);
		}
	}

	/**
	 * @author urs
	 * 
	 */
	public class Property_Dummy {
		private String name;
		private String value;

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getValue() {
			return value;
		}

		public void setValue(String value) {
			this.value = value;
		}
	}

	private String filename;
	private String callablename;
	private String skiplist;
	private boolean clearconsole = true;

	private List<Property_Dummy> properties = new ArrayList<Atlfow.Property_Dummy>();
	private boolean chaincalls = true;
	private List<String> skipList = new ArrayList<String>();

	/**
	 * 	
	 */
	public Property_Dummy createProperty() {
		Property_Dummy p = new Property_Dummy();
		properties.add(p);
		return p;
	}

	@Override
	public void execute() throws BuildException {
		Calls calls = CallchainLaunchConfigurationDelegate.loadCallchain(filename);
		if (calls == null)
			throw new BuildException("Could not load atlfow model, file was :" + filename);
		Callable callableToStart = CallchainLaunchConfigurationDelegate.findCallable(callablename, calls);
		if (callableToStart == null)
			throw new BuildException("Could find the callable to start, callable name was :" + callablename);

		if (skiplist != null) {
			String[] split = skiplist.split(",");
			for (int i = 0; i < split.length; i++) {
				String string = split[i];
				skipList.add(string);
			}
		}

		if (clearconsole)
			CallchainLaunchConfigurationDelegate.clearConsole();
		
		AntLogMonitor monitor = new AntLogMonitor();
		
		MCcallchain ccallchain = new MCcallchain();
		try {
			applyPropertyValues(calls);
			if (!chaincalls)
				ccallchain.startSingleTransformation(callableToStart,monitor);
			else
				ccallchain.executeTransformations(callableToStart, skipList,monitor);

		} catch (Exception e) {
			throw new BuildException(e);
		}
	}

	/**
	 * Set the values to the properties.
	 * 
	 * @param calls
	 */
	private void applyPropertyValues(Calls calls) {
		EList<Property> properties = calls.getProperties();

		for (Property_Dummy pd : this.properties) {
			for (Property property : properties) {
				if (property.getName() != null)
					if (property.getName().equals(pd.name))
						property.setValue(pd.value);
			}
		}
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public void setCallablename(String callablename) {
		this.callablename = callablename;
	}

	public void setSkiplist(String skiplist) {
		this.skiplist = skiplist;
	}

	public void setClearconsole(boolean clearconsole) {
		this.clearconsole = clearconsole;
	}

	public void setChaincalls(boolean chaincalls) {
		this.chaincalls = chaincalls;
	}
}
