/*
 * (c) urs zeidler
 */
package de.urszeidler.eclipse.callchain.lauching.core;

import static de.urszeidler.eclipse.callchain.lauching.core.CallchainConfiguration.CONTAINER;
import static de.urszeidler.eclipse.callchain.lauching.core.CallchainConfiguration.NONE;
import static de.urszeidler.eclipse.callchain.lauching.core.CallchainConfiguration.RESOURCE;
import static de.urszeidler.eclipse.callchain.lauching.core.CallchainConfiguration.STRING;
import static de.urszeidler.eclipse.callchain.lauching.core.CallchainConfiguration.URI;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.model.ILaunchConfigurationDelegate;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.presentation.EcoreActionBarContributor.ExtendedLoadResourceAction.ExtendedLoadResourceDialog;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.TextConsole;
import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
import org.eclipse.ui.dialogs.ISelectionStatusValidator;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import de.urszeidler.eclipse.callchain.Call;
import de.urszeidler.eclipse.callchain.Callable;
import de.urszeidler.eclipse.callchain.Calls;
import de.urszeidler.eclipse.callchain.ExternalCallable;
import de.urszeidler.eclipse.callchain.Generator;
import de.urszeidler.eclipse.callchain.Property;
import de.urszeidler.eclipse.callchain.Switch;
import de.urszeidler.eclipse.callchain.executors.MCcallchain;

/**
 * <p>
 * CallchainLaunchConfigurationDelegate
 * </p>
 * Created : 03.06.2008
 * 
 * @author urs
 * @version $Revision$, $Id$
 */
public class CallchainLaunchConfigurationDelegate implements
		ILaunchConfigurationDelegate {

	private static final Object ATL_CONSOLE = "org.eclipse.m2m.atl.adt.editor.console";
	private Map<String, PropertyHolder> map = new HashMap<String, PropertyHolder>();

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.debug.core.model.ILaunchConfigurationDelegate#launch(org.
	 * eclipse.debug.core.ILaunchConfiguration, java.lang.String,
	 * org.eclipse.debug.core.ILaunch,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 */
	@SuppressWarnings("unchecked")
	public void launch(ILaunchConfiguration configuration, String mode,
			ILaunch launch, IProgressMonitor monitor) throws CoreException {

		String callchainUri = configuration.getAttribute(
				CallchainConfiguration.CALLCHAINURI, "");
		String callableName = configuration.getAttribute(
				CallchainConfiguration.CALLABLENAME, "");
		boolean chaincalls = configuration.getAttribute(
				CallchainConfiguration.CHAINCALLS, false);
		boolean clearConsole = configuration.getAttribute(
				CallchainConfiguration.CLEARCONSOLE, true);
		Calls calls = loadCallchain(callchainUri);
		Callable callableToStart = findCallable(callableName, calls);
		List<String> list = configuration.getAttribute(
				CallchainConfiguration.SKIPPLIST, new ArrayList<String>());

		if (configuration.getAttributes().containsKey(
				CallchainConfiguration.PROPERTIESNAMES)) {
			List<?> plist = configuration.getAttribute(
					CallchainConfiguration.PROPERTIESNAMES,
					new ArrayList<String>());
			for (Object object : plist) {
				PropertyHolder propertyHolder = map.get(object.toString());
				if (propertyHolder == null)
					propertyHolder = createPropertyHolder(object.toString());

				propertyHolder.setValue(configuration.getAttribute(
						propertyHolder.getName()
								+ CallchainConfiguration.PROPERTY_VALUE,
						(String) null));
				propertyHolder.setDialogType(configuration.getAttribute(
						propertyHolder.getName()
								+ CallchainConfiguration.PROPERTY_EDITOR_TYPE,
						(String) null));
				map.put(propertyHolder.getName(), propertyHolder);
			}

		}

		applyPropertyValues(calls);

		if (clearConsole)
			clearConsole();

		if (callableToStart == null)
			throw new CoreException(new Status(IStatus.ERROR, "",
					"no callable To Start found :" + callableName));

		MCcallchain ccallchain = new MCcallchain();
		if (!chaincalls)
			ccallchain.startSingleTransformation(callableToStart);
		else
			ccallchain.executeTransformations(callableToStart, list);

	}

	/**
	 * Set the values to the properties.
	 * 
	 * @param calls
	 */
	private void applyPropertyValues(Calls calls) {
		EList<Property> properties = calls.getProperties();
		for (Property property : properties) {
			PropertyHolder propertyHolder = map.get(property.getName());
			if (propertyHolder != null) {
				if (propertyHolder.getDialogType() != null) {
					String dialogValue = getDialogValue(propertyHolder);
					property.setValue(dialogValue);
				} else if (propertyHolder.getValue() != null) {
					property.setValue(propertyHolder.getValue());
				}
			}
		}
	}

	/**
	 * Displays the dialog for the property.
	 * 
	 * @param propertyHolder
	 * @return
	 */
	private String getDialogValue(final PropertyHolder propertyHolder) {
		if (propertyHolder.getDialogType().equals(NONE)) {
			return propertyHolder.getValue();
		}
		final String[] value = new String[] { "" };

		if (propertyHolder.getDialogType().equals(STRING)) {
			Display.getDefault().syncExec(new Runnable() {
				public void run() {

					Shell shell = Display.getCurrent().getActiveShell();

					String initialValue = "";
					if (propertyHolder.getProperty() != null)
						if (propertyHolder.getProperty().getDefaultValue() != null)
							initialValue = propertyHolder.getProperty()
									.getDefaultValue();

					InputDialog dialog = new InputDialog(shell,
							"Define Property",
							"Define the value for the property "
									+ propertyHolder.getName(), initialValue,
							null);

					if (dialog.open() == Dialog.OK) {
						value[0] = dialog.getValue();
					}

				}
			});

		}

		if (propertyHolder.getDialogType().equals(RESOURCE)) {
			Display.getDefault().syncExec(new Runnable() {
				public void run() {

					Shell shell = Display.getCurrent().getActiveShell();

					String initialValue = "";
					if (propertyHolder.getProperty() != null)
						if (propertyHolder.getProperty().getDefaultValue() != null)
							initialValue = propertyHolder.getProperty()
									.getDefaultValue();

					ElementTreeSelectionDialog dialog = new ElementTreeSelectionDialog(
							shell, new WorkbenchLabelProvider(),
							new WorkbenchContentProvider());
					dialog.setInput(ResourcesPlugin.getWorkspace().getRoot());
					dialog.setMessage("Define the value for the property "
							+ propertyHolder.getName());
					dialog.setTitle("Select a file");
					dialog.setAllowMultiple(false);
					dialog.setDoubleClickSelects(true);
					dialog.setValidator(new ISelectionStatusValidator() {
						public IStatus validate(Object[] selection) {
							IStatus ret = Status.CANCEL_STATUS;
							if (selection.length == 1) {
								if (selection[0] instanceof IFile) {
									ret = Status.OK_STATUS;
								}
							}
							return ret;
						}
					});
					Path p = new Path(initialValue);
					dialog.setInitialSelection(p);

					if (dialog.open() == Dialog.OK) {
						Object result = dialog.getFirstResult();
						if (result instanceof IFile) {
							IFile file = (IFile) result;
							value[0] = file.getFullPath().toString();
						}
					}

				}
			});

		}

		if (propertyHolder.getDialogType().equals(CONTAINER)) {
			Display.getDefault().syncExec(new Runnable() {
				public void run() {
					Shell shell = Display.getCurrent().getActiveShell();

					String initialValue = "";
					if (propertyHolder.getProperty() != null)
						if (propertyHolder.getProperty().getDefaultValue() != null)
							initialValue = propertyHolder.getProperty()
									.getDefaultValue();

					ElementTreeSelectionDialog dialog = new ElementTreeSelectionDialog(
							shell, new WorkbenchLabelProvider(),
							new WorkbenchContentProvider());
					dialog.setInput(ResourcesPlugin.getWorkspace().getRoot());
					dialog.setTitle("Select a container");
					dialog.setMessage("Define the value for the property "
							+ propertyHolder.getName());
					dialog.setAllowMultiple(false);
					dialog.setDoubleClickSelects(true);
					dialog.addFilter(new ViewerFilter() {

						@Override
						public boolean select(Viewer viewer,
								Object parentElement, Object element) {
							if (element instanceof IContainer)
								return true;

							return false;
						}
					});
					Path p = new Path(initialValue);
					dialog.setInitialSelection(p);

					if (dialog.open() == Dialog.OK) {
						Object result = dialog.getFirstResult();
						if (result instanceof IContainer) {
							IContainer container = (IContainer) result;
							value[0] = container.getFullPath().toString();
						}
					}

				}
			});

		}

		if (propertyHolder.getDialogType().equals(URI)) {
			Display.getDefault().syncExec(new Runnable() {
				public void run() {
					Shell shell = Display.getCurrent().getActiveShell();
					ExtendedLoadResourceDialog dialog = new ExtendedLoadResourceDialog(
							shell, (EditingDomain) null);
					if (dialog.open() == Dialog.OK) {
						value[0] = dialog.getURIText();
					}

				}
			});

		}

		return value[0];
	}

	/**
	 * clears the atl console
	 */
	public static void clearConsole() {
		IConsoleManager consoleManager = org.eclipse.ui.console.ConsolePlugin
				.getDefault().getConsoleManager();
		IConsole[] consoles = consoleManager.getConsoles();
		for (int i = 0; i < consoles.length; i++) {
			IConsole console = consoles[i];
			if (console.getName().equals(ATL_CONSOLE)) {
				if (console instanceof TextConsole) {
					TextConsole tc = (TextConsole) console;
					tc.clearConsole();
				}
			}

		}
	}

	/**
	 * 
	 * @param callableName
	 * @param calls
	 * @return
	 */
	public static Callable findCallable(String callableName, Calls calls) {
		if (callableName == null | "".equals(callableName))
			return null;

		callableName = callableName.trim();
		EList<Call> list = calls.getCall();
		// Callable callableToStart = null;
		for (Call call : list) {
			if (call.getName() != null)
				if (callableName.endsWith(call.getName())) {
					return call;

				}
		}
		EList<Generator> generators = calls.getGenerators();
		for (Generator generator : generators) {
			if (generator.getName() != null)
				if (callableName.endsWith(generator.getName())) {
					return generator;
				}
		}

		EList<Switch> switches = calls.getSwitches();
		for (Switch generator : switches) {
			if (generator.getName() != null)
				if (callableName.endsWith(generator.getName())) {
					return generator;
				}
		}

		EList<ExternalCallable> externalCallables = calls.getExternCallables();
		for (ExternalCallable generator : externalCallables) {
			if (generator.getName() != null)
				if (callableName.endsWith(generator.getName())) {
					return generator;
				}
		}

		return null;
	}

	public static Calls loadCallchain(String callchainUri) {
		File file = new File(callchainUri);
		URI uri = file.isFile() ? org.eclipse.emf.common.util.URI
				.createFileURI(file.getAbsolutePath())
				: org.eclipse.emf.common.util.URI.createPlatformResourceURI(
						callchainUri, false);

		try {
			ResourceSetImpl resourceSet = new ResourceSetImpl();
			// Demand load resource for this file.
			//
			Resource resource = resourceSet.getResource(uri, true);
			return (Calls) resource.getContents().get(0);
			// Validate the contents of the loaded resource.
			//

		} catch (RuntimeException exception) {

		}

		return null;

	}

	/**
	 * create a propety holder for the property.
	 * 
	 * @param property
	 * @return
	 */
	public static PropertyHolder createPropertyHolder(Property property) {
		PropertyHolder propertyHolder = createPropertyHolder(property.getName());
		propertyHolder.setProperty(property);
		return propertyHolder;
	}

	/**
	 * @param name
	 * @return
	 */
	public static PropertyHolder createPropertyHolder(String name) {
		PropertyHolder propertyHolder = new PropertyHolder();
		propertyHolder.setName(name);
		return propertyHolder;
	}

}
