package de.urszeidler.eclipse.callchain.topcasedgenerators.preferences;

import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import de.urszeidler.eclipse.callchain.editor.preferences.UriFieldEditor;
import de.urszeidler.eclipse.callchain.topcasedgenerators.Activator;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class TopcasedUriPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public TopcasedUriPreferencePage() {
		super(GRID);
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		setDescription("The preferences of the topcased generators.");
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		addField(new UriFieldEditor(PreferenceConstants.UML_IMPORTER_URIS, "UML uri", "add",
				getFieldEditorParent()));
		addField(new UriFieldEditor(PreferenceConstants.PROPERTIES_URIS, "Tabbed Properties generator", "add",
				getFieldEditorParent()));
		addField(new UriFieldEditor(PreferenceConstants.DIAGRAM_URIS, "Diagram generator uris", "add",
				getFieldEditorParent()));
		addField(new UriFieldEditor(PreferenceConstants.EDITOR_URIS, "Editor generator uris", "add",
				getFieldEditorParent()));
		addField(new UriFieldEditor(PreferenceConstants.DIAGRAM_EXCHANGE_URIS, "Diagram exchange uris", "add",
				getFieldEditorParent()));
		addField(new UriFieldEditor(PreferenceConstants.DIAGRAM_EXCHANGE_KEEP_TYPES, "Diagram exchange uris", "add",
				getFieldEditorParent()));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}
	
}