/*******************************************************************************
 * urs zeidler
 ******************************************************************************/
package de.urszeidler.eclipse.callchain.ui.wizards;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;

/**
 * @generated
 */
public class CallchainSimpleTemplatePage extends WizardPage {
	protected static final String BROWSE_TEXT = "...";

	private static final String[] KNOWN_ATL_NATURES = {
			"org.atl.eclipse.adt.builder.atlNature (old)",
			"org.eclipse.m2m.atl.adt.builder.atlNature" };

	/**
	 * Contains the resource that is initially selected when the wizard is
	 * opened. In the case of a file with an invalid extension, we store the
	 * container of the file. This variable is then used to load the
	 * corresponding data in the wizard.
	 */
	protected IResource selectedResource;

	// private boolean allowInit;

	// SWT objects
	// private Button existingModelBt;

	// private Button newModelBt;

	private Group newModelGrp;

	protected Text outputPath;

	private Button outputPathBt;

	protected Text nameFd;

	// private ComboViewer templateCbViewer;

	// private Group existingModelGrp;

	// private Text modelFd;

	// private Button modelBt;

	// private TreeViewer viewer;

	// private ComboViewer rootDiagramCbViewer;

	// private Button initializeBt;

	private ModifyListener validationListener = new ModifyListener() {
		public void modifyText(ModifyEvent e) {
			setPageComplete(validatePage());
		}
	};

	private Button genmodelBt;

	private String fileExtension;

	private String sourceFileExtension;

	private String projectName;

	private Group basicGroup;

	private Button checkBox_addAtlNature;

	private Combo natureTypeCombobox;

	private boolean addAtlNature = false;

	private int natureType = 1;

	protected String selectedFile;

	/**
	 * Constructor
	 * 
	 * @param pageName
	 *            the page title
	 * @param selection
	 *            the selected object
	 * @param allowDiagramInitialization
	 */
	public CallchainSimpleTemplatePage(String pageName,
			IStructuredSelection selection) {
		super(pageName);
		setPageComplete(false);

		Object sel = null;
		if (selection != null)
			sel = selection.getFirstElement();

		if (sel != null && sel instanceof IResource) {
			if (sel instanceof IFile) {
				// check if the selected file as the expected extension
				if (((IFile) sel).getFileExtension().equals(
						getSourceFileExtension())) {
					selectedResource = (IFile) sel;
				} else {
					selectedResource = ((IResource) sel).getParent();
				}
			} else {
				selectedResource = (IResource) sel;
			}
		} else {
			selectedResource = ResourcesPlugin.getWorkspace().getRoot();
		}

		// this.allowInit = false;
	}

	public CallchainSimpleTemplatePage(String pageName,
			IStructuredSelection selection, String sourceFileExtension) {
		super(pageName);
		setPageComplete(false);
		setSourceFileExtension(sourceFileExtension);

		Object sel = selection.getFirstElement();
		if (sel != null && sel instanceof IResource) {
			if (sel instanceof IFile) {
				// check if the selected file as the expected extension
				if (((IFile) sel).getFileExtension().equals(
						getSourceFileExtension())) {
					selectedResource = (IFile) sel;
				} else {
					selectedResource = ((IResource) sel).getParent();
				}
			} else {
				selectedResource = (IResource) sel;
			}
		} else {
			selectedResource = ResourcesPlugin.getWorkspace().getRoot();
		}
	}

	private String getSourceFileExtension() {

		return sourceFileExtension;
	}

	/**
	 * Creation of the wizard page
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite mainComp = new Composite(parent, SWT.NONE);
		mainComp.setLayout(new GridLayout());
		mainComp.setLayoutData(new GridData(GridData.FILL_BOTH));
		mainComp.setFont(parent.getFont());

		createDestinationGroup(mainComp);

		// Add listeners
		hookListeners();

		// Initialize widgets value
		loadData();

		// Show description on opening
		setErrorMessage(null);
		setMessage(null);
		setControl(mainComp);

		setPageComplete(validatePage());
	}

	protected void loadData() {
		if (selectedResource instanceof IFile) {
			// existingModelBt.setSelection(true);
			// newModelGrp.setEnabled(false);
			IFile file = (IFile) selectedResource;
			selectedFile = file.getFullPath().toString();
			outputPath.setText(selectedResource.getParent().getFullPath()
					.toString());
			setnameFd(file);

		} else if (selectedResource instanceof IContainer) {
			IContainer container = (IContainer) selectedResource;
			outputPath.setText(container.getFullPath().toString());

		}

	}

	/**
	 * @param file
	 */
	private void setnameFd(IFile file) {
		setnameFdText(file.getName().substring(0,
				file.getName().length() - file.getFileExtension().length() - 1));

		projectName = file.getProject().getName();
	}

	/**
	 * set the text in the text field
	 * 
	 * @param text
	 */
	protected void setnameFdText(String text) {
		nameFd.setText(text);
	}

	/**
	 * Create the Destination group
	 * 
	 * @param parent
	 *            the Composite
	 */
	protected void createDestinationGroup(Composite parent) {

		createNewModelGroup(parent);
		createProjectNatureGoup(parent);

	}

	protected void createProjectNatureGoup(Composite parent) {
		basicGroup = new Group(parent, SWT.NONE);
		basicGroup.setLayout(new GridLayout(3, false));
		basicGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		basicGroup.setText("Project Nature");

		checkBox_addAtlNature = new Button(basicGroup, SWT.CHECK);
		checkBox_addAtlNature.setText("Add Atl Nature");
		checkBox_addAtlNature.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				addAtlNature = checkBox_addAtlNature.getSelection();
			}
		});

		Label label = new Label(basicGroup, SWT.NONE);
		label = new Label(basicGroup, SWT.NONE);

		label = new Label(basicGroup, SWT.NONE);
		label.setText("nature type");
		natureTypeCombobox = new Combo(basicGroup, SWT.DROP_DOWN);
		natureTypeCombobox.setItems(KNOWN_ATL_NATURES);
		natureTypeCombobox.setText(KNOWN_ATL_NATURES[1]);
		natureTypeCombobox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				natureType = natureTypeCombobox.getSelectionIndex();
			}
		});
		// natureTypeCombobox.addModifyListener(modifyListener);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		natureTypeCombobox.setLayoutData(gd);

	}

	protected void createNewModelGroup(Composite parent) {
		newModelGrp = new Group(parent, SWT.NONE);
		newModelGrp.setLayout(new GridLayout(3, false));
		newModelGrp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// The destination directory
		Label directoryLbl = new Label(newModelGrp, SWT.NONE);
		directoryLbl.setText("Directory : ");

		outputPath = new Text(newModelGrp, SWT.BORDER);
		outputPath.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		outputPath.setEnabled(false);

		outputPathBt = new Button(newModelGrp, SWT.PUSH);
		outputPathBt.setText(BROWSE_TEXT);

		// The new name of the model file to create
		Label nameLbl = new Label(newModelGrp, SWT.NONE);
		nameLbl.setText("Input Model name : ");

		nameFd = new Text(newModelGrp, SWT.BORDER);
		GridData layoutNameFd = new GridData(GridData.FILL_HORIZONTAL);
		// layoutNameFd.horizontalSpan = 2;
		nameFd.setLayoutData(layoutNameFd);
		genmodelBt = new Button(newModelGrp, SWT.PUSH);
		genmodelBt.setText(BROWSE_TEXT);

	}

	/**
	 * Add the listeners to the Buttons and the Text widgets
	 */
	private void hookListeners() {
		outputPath.addModifyListener(validationListener);
		nameFd.addModifyListener(validationListener);
		// Button listeners
		outputPathBt.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleDirectoryChoose();
			}
		});

		genmodelBt.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleGenmodelChoose();
			}
		});

	}

	protected void handleGenmodelChoose() {
		String initialDirectory = ResourcesPlugin.getWorkspace().getRoot()
				.getFullPath().toString();

		//TODO : make the text of the dialog configureable
		ResourceSelectionDialog dialog = new ResourceSelectionDialog(
				getShell(), ResourcesPlugin.getWorkspace().getRoot()
						.findMember(initialDirectory),
				"Choose an existing model file :");

		if (dialog.open() == Window.OK) {
			Object[] results = dialog.getResult();
			if (results.length == 1 && results[0] instanceof IFile) {
				IFile newPath = (IFile) results[0];
				setnameFd(newPath);
				outputPath.setText(newPath.getParent().toString());
				projectName = newPath.getProject().getName();
			}
		}

	}

	/**
	 * Handle the directory choice button action
	 */
	protected void handleDirectoryChoose() {
		// String initialDirectory = ResourcesPlugin.getWorkspace().getRoot()
		// .getFullPath().toString();
		IResource resource = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(outputPath.getText());
		IContainer container = ResourcesPlugin.getWorkspace().getRoot();
		if (resource instanceof IContainer) {
			container = (IContainer) resource;

		}// end of if (resource instanceof IContainer)
		//TODO : make the text of the dialog configureable
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(
				getShell(), container, true, "Choose an existing container :");
		if (dialog.open() == Window.OK) {
			Object[] results = dialog.getResult();
			if (results.length == 1 && results[0] instanceof IPath) {
				IPath newPath = (IPath) results[0];
				outputPath.setText(newPath.toString());
				if (results[0] instanceof IContainer) {
					container = (IContainer) results[0];
					projectName = container.getProject().getName();
				}// end of if (resource instanceof IContainer)

			}
		}
	}

	// /**
	// * Handle the model choice button action
	// */
	// protected void handleModelChoose() {
	// // The initial directory is always the Workspace
	// String initialDirectory = ResourcesPlugin.getWorkspace().getRoot()
	// .getFullPath().toString();
	//
	// ResourceSelectionDialog dialog = new ResourceSelectionDialog(
	// getShell(), ResourcesPlugin.getWorkspace().getRoot()
	// .findMember(initialDirectory),
	// "Choose an existing model file :");
	// if (dialog.open() == Window.OK) {
	// Object[] results = dialog.getResult();
	// if (results.length == 1 && results[0] instanceof IFile) {
	// // TODO :
	// nameFd.setText(((IFile)results[0]).getName());
	// // results[0]).getFullPath().toString());
	// //
	// // // update the other widgets
	// // loadModelFile();
	// }
	// }
	// }

	/**
	 * Return the ResourceSet used to store the model and diagram file. By
	 * default create a new ResourceSet.
	 * 
	 * @return ResourceSet
	 */
	protected ResourceSet getResourceSet() {
		return new ResourceSetImpl();
	}

	/**
	 * Check if the informations contained in the page are OK
	 * 
	 * @return true is OK
	 */
	protected boolean validatePage() {
		// erase all previous messages
		setMessage(null);
		setErrorMessage(null);
		return validateNewModelGroup();
	}

	/**
	 * Validate the group if the new model option is chosen
	 * 
	 * @return <code>true</code> if data is valid
	 */
	private boolean validateNewModelGroup() {

		if (getModelName() == null || "".equals(getModelName())) {
			setErrorMessage("Model name cannot be empty.");
			return false;
		} else {
			validateExistingFiles();
		}

		return true;
	}

	/**
	 * Check if the created files does not already exist
	 */
	protected void validateExistingFiles() {
		// TODO : think of an other way to check
		// if (modelFileExist()) {
		// if (diagramFileExist()) {
		// setMessage(
		// "The model file and the diagram file already exist. They will be overwritten !"
		// ,
		// IMessageProvider.WARNING);
		// }
		// // else {
		// // setMessage(
		// // "The model file already exists. It will be overwritten !",
		// // IMessageProvider.WARNING);
		// // }
		// } else if (diagramFileExist()) {
		// setMessage(
		// "The diagram file already exists. It will be overwritten !",
		// IMessageProvider.WARNING);
		// }
	}

	protected boolean modelFileExist() {
		// if (isNewModel())
		// {
		String path = outputPath.getText() + File.separator
				+ getModelNameWithoutExtension(nameFd.getText()) + "."
				+ getFileExtension();
		IResource resource = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(path);
		return resource != null;
	}

	protected boolean diagramFileExist() {
		// if (isNewModel())
		// {
		String path = outputPath.getText() + File.separator
				+ getModelNameWithoutExtension(nameFd.getText()) + "."
				+ getFileExtension() + "_diagram";
		IResource resource = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(path);
		return resource != null;
	}

	/**
	 * Returns if the diagram must be initialized with model objects
	 * 
	 * @return <code>true</code> if diagram must be initialized
	 */
	public boolean isInitialized() {
		// TODO:
		// if (allowInit)
		// {
		// return ModelerPlugin.getDefault().getPreferenceStore().getBoolean(
		// ModelerPreferenceConstants.P_INITIALIZE_DIAGRAM);
		// }

		return false;
	}

	/**
	 * Returns the selected destination container
	 * 
	 * @return the selected container
	 */
	public IContainer getSelectedIContainer() {
		IContainer container = null;
		IResource resource = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(outputPath.getText());
		if (resource instanceof IContainer) {
			container = (IContainer) resource;
			return container;
		}// end of if (resource instanceof IContainer)

		return ResourcesPlugin.getWorkspace().getRoot();
	}

	public String getModelFile() {
		return selectedFile;
	}

	/**
	 * Returns the selected model name
	 * 
	 * @return the model name. Returns null if no model file name is set.
	 */
	public String getModelName() {
		// if (isNewModel())
		// {
		if (nameFd != null && !nameFd.isDisposed()) {
			return getModelNameWithoutExtension(nameFd.getText());
		}
		// }
		// else
		// {
		// if (modelFd != null && !modelFd.isDisposed())
		// {
		// IResource resource =
		// ResourcesPlugin.getWorkspace().getRoot().findMember(modelFd.getText())
		// ;
		// return getModelNameWithoutExtension(resource.getName());
		// }
		// }
		return null;
	}

	// /**
	// * Returns the root EObject of the model file
	// *
	// * @return the EObject
	// */
	// public EObject getRootEObject()
	// {
	// return (EObject) ((EList) viewer.getInput()).get(0);
	// }

	// /**
	// * Returns the selected EObject that will be associated with the diagrams
	// * file. If no object is selected, returns the rootEObject of the model
	// *
	// * @return the EObject
	// */
	// public EObject getDiagramEObject()
	// {
	// if (((IStructuredSelection) viewer.getSelection()).getFirstElement() ==
	// null)
	// {
	// return getRootEObject();
	// }
	// else
	// {
	// return (EObject)
	// AdapterFactoryEditingDomain.unwrap(((IStructuredSelection)
	// viewer.getSelection()).getFirstElement());
	// }
	// }

	/**
	 * Returns the name of the modelFile without extension
	 * 
	 * @param fullName
	 *            the name with (or without) extension
	 * @return the name of the modelFile without extension
	 */
	protected String getModelNameWithoutExtension(String fullName) {
		String[] nameSplitted = fullName.split("." + getFileExtension());
		return nameSplitted[0];
	}

	// /**
	// * @see org.topcased.modeler.wizards.DiagramsPage#getEditorID()
	// * @generated
	// */
	// public String getEditorID() {
	// return "de.urszeidler.eclipse.MBcallchainEd.ed.editor.CallchainEditor";
	// }

	/**
	 * @see org.topcased.modeler.wizards.DiagramsPage#getFileExtension()
	 * @generated
	 */
	public String getFileExtension() {
		return fileExtension;
	}

	/**
	 * @see org.topcased.modeler.wizards.DiagramsPage#getAdapterFactory()
	 */
	@SuppressWarnings("unchecked")
	public ComposedAdapterFactory getAdapterFactory() {
		List factories = new ArrayList();
		factories
				.add(new de.urszeidler.eclipse.callchain.provider.CallchainItemProviderAdapterFactory());
		factories.add(new ResourceItemProviderAdapterFactory());
		factories.add(new ReflectiveItemProviderAdapterFactory());

		return new ComposedAdapterFactory(factories);
	}

	public void setFileExtension(String fileExtension) {
		this.fileExtension = fileExtension;
	}

	public void setSourceFileExtension(String sourceFileExtension) {
		this.sourceFileExtension = sourceFileExtension;
	}

	public String getProjectName() {
		return projectName;
	}

	public int getNatureType() {
		return natureType;
	}

	public boolean isAddAltNature() {
		return addAtlNature;
	}

	/**
	 * @return the nameFd
	 */
	public Text getNameFd() {
		return nameFd;
	}

}
