/**
 * 
 */
package de.urszeidler.eclipse.callchain.util;

import java.util.regex.Pattern;

import org.eclipse.emf.common.util.EList;

import de.urszeidler.eclipse.callchain.Property;

/**
 * A helper class for property substitution.
 * 
 * @author urs
 * 
 */
public class CallsUtil {

	private static final String PROPERTY_SUFFIX = "}";
	private static final String PROPERTY_PREFIX = "${";
	private static final Pattern propertyTester = Pattern.compile(".*\\$\\{.*\\}.*");

	/**
	 * Substitutes all property expressions in the given string.
	 * 
	 * @param uri
	 * @param properties
	 * @return
	 */
	public static String doSubstitude(String uri, EList<Property> properties) {
		String temp = uri;

		while (containsProperty(temp)) {
			String oldTemp = temp;
			temp = substitude(temp, properties);
			if (oldTemp.equals(temp))
				break;
		}
		return temp;
	}

	private static String substitude(String temp, EList<Property> properties) {
		if (temp == null)
			return null;

		for (Property property : properties) {
			if (temp.contains(toPropertyExpression(property))) {
				String value = property.getValue();
				if (toPropertyExpression(property) == null || value == null) {
					continue;
				}
				temp = temp.replace(toPropertyExpression(property), value);
			}
		}
		return temp;
	}

	private static String toPropertyExpression(Property property) {
		return PROPERTY_PREFIX + property.getName() + PROPERTY_SUFFIX;
	}

	public static boolean containsProperty(String temp) {
		if (temp == null)
			return false;

		return propertyTester.matcher(temp).matches();
	}

}
