/**
 * 
 */
package de.urszeidler.eclipse.callchain.ui.wizards.templates;

import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;
import org.osgi.framework.Bundle;

import de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard;
import de.urszeidler.eclipse.callchain.ui.wizards.WizardsPlugin;
import de.urszeidler.eclipse.callchain.wizards.template.preferences.PreferenceConstants;

/**
 * @author urs
 * 
 */
public class CreateDefaultAtlFlowProject extends BasicTemplateWizard {

	private static final String ATL_PROJECT_NAME = WizardsPlugin.getDefault().getPreferenceStore()
			.getString(PreferenceConstants.P_TEMPLATE_PROJECT_PATH);// "atl flow transformations";
	private static final String GMF_TEMPLATE_PATH = "/templates/GMF_from_ECORE";
	private static final String EEF_TEMPLATE_PATH = "/templates/eef-process";
	private static final String SIMPLE_ECORE_PATH = "/templates/simple_ecore";
	private static final String ATLCOPY_TEMPLATE_PATH = "/templates/mapping2atl";
	private static final String TOPCASE_DIAGRAM_TEMPLATE_PATH = "/templates/Editor_and_Diagram_from_Genmodel";
	private static final String TOPCASE_PROPERTY_TEMPLATE_PATH = "/templates/Topcased_PropertiesView";
	protected static final String EMF_TEMPLATE_PATH = "/templates/simple_ecore";
	protected static final String EMF_GENERIC_TEMPLATE_PATH = "/templates/emf_generic";
	protected static final String CALLCHAIN_REPORT_TEMPLATE_PATH_SRC = "/templates/callchainReport/acceleo";
	protected static final String CALLCHAIN_REPORT_TEMPLATE_PATH = "/templates/callchainReport/callchainReport";
	private AtlFlowProjectPage projectPage;
	private String createdFile;
	private Set<String> excludeFiles_gmfTemplates = new HashSet<String>();
	private IProject project;
	private String modelname = null;
	private String baseModelPath=null;
	private String modelFile =null;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 * org.eclipse.jface.viewers.IStructuredSelection)
	 */
	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		// excludeFiles_gmfTemplates.add("createGMFmapping.callchain");
		// excludeFiles_gmfTemplates.add("createGMFmapping.callchain_diagram");
		excludeFiles_gmfTemplates.add("[default] createMappin_%name%.launch");
		excludeFiles_gmfTemplates.add("[startpoint] bootstrapToolTemplate_%name%.launch");
		excludeFiles_gmfTemplates.add("ecore2gmfTool_custom_%name%.atl");

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#getModelName
	 * ()
	 */
	@Override
	protected String getModelName() {
		return modelname;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#
	 * getBaseTemplatePath()
	 */
	@Override
	protected String getBaseTemplatePath() {
		return baseModelPath;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#
	 * getTargetContainer()
	 */
	@Override
	protected IContainer getTargetContainer() {
		return project;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#getModelFile
	 * ()
	 */
	@Override
	protected String getModelFile() {
		return modelFile;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.urszeidler.eclipse.callchain.ui.wizards.BasicTemplateWizard#getProjectName
	 * ()
	 */
	@Override
	protected String getProjectName() {
		return ATL_PROJECT_NAME;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		final boolean addAltNature = projectPage.isAddAltNature();
		final int natureType = projectPage.getNatureType();
		final boolean canCreateTopcasedView = projectPage.canCreateTopcasedView();
		final boolean canCreateGMF = projectPage.canCreateGMF();
		final boolean canCreateEEF = projectPage.canCreateEEF();
		final boolean canCreateAtlCopy = projectPage.canCreateAtlCopy();
		final boolean canCreateTopcasedDiagram = projectPage.canCreateTopcasedDiagram();
		final boolean canCreateEcore = projectPage.canCreateEcore();
		final boolean canCreateGenericEmf = projectPage.canCreateEcore();
		final boolean canCreateCallchainReport = projectPage.canCreateCallchainReport();

		if (projectPage.isPageComplete()) {

			IRunnableWithProgress op = new IRunnableWithProgress() {

				@Override
				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {

					if (monitor != null) {
						monitor.beginTask("creating templates", 6);
						if (monitor.isCanceled())
							return;
					}
					try {
						boolean result = true;
						createProject(monitor);
						if (monitor != null) {
							monitor.worked(1);
							if (monitor.isCanceled())
								return;
						}
						if (canCreateGMF) {
							result = result
									& createFilesFromTemplate(GMF_TEMPLATE_PATH, excludeFiles_gmfTemplates,
											"ecore_to_gmf", monitor);
							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}
						}
						if (canCreateEEF) {
							result = result
									& createFilesFromTemplate(EEF_TEMPLATE_PATH, Collections.EMPTY_SET, "eef-process",
											monitor);
							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}
						}
						if (canCreateAtlCopy) {
							result = result
									& createFilesFromTemplate(ATLCOPY_TEMPLATE_PATH, Collections.EMPTY_SET, "atlcopy",
											monitor);
							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}
						}
						if (canCreateEcore) {
							result = result
									& createFilesFromTemplate(EMF_GENERIC_TEMPLATE_PATH, Collections.EMPTY_SET,
											"emf_generic", monitor);
							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}

						}
						if (canCreateCallchainReport) {

							result = result
									& createFilesFromTemplate(CALLCHAIN_REPORT_TEMPLATE_PATH_SRC, Collections.EMPTY_SET,
											"../", monitor);
							
							
							result = result
									& createFilesFromTemplate(CALLCHAIN_REPORT_TEMPLATE_PATH, Collections.EMPTY_SET,
											"callchainReport", monitor);
							addAcceleoNature(monitor);

							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}

						}
						if (canCreateGenericEmf) {
							result = result
									& createFilesFromTemplate(EMF_TEMPLATE_PATH, Collections.EMPTY_SET, "simple_ecore",
											monitor);
							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}
						}
						if (canCreateTopcasedDiagram) {
							result = result
									& createFilesFromTemplate(TOPCASE_DIAGRAM_TEMPLATE_PATH, Collections.EMPTY_SET,
											"topcased_diagram", monitor);
							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}
						}
						if (canCreateTopcasedView) {
							result = result
									& createFilesFromTemplate(TOPCASE_PROPERTY_TEMPLATE_PATH, Collections.EMPTY_SET,
											"topcase_property_page", monitor);
							if (monitor != null) {
								monitor.worked(1);
								if (monitor.isCanceled())
									return;
							}
						}
						if (addAltNature) {
							addATLnature(null, natureType);
						}
					} catch (Exception e) {
						WizardsPlugin.log(e);
					}
				}
			};

			try {
				getContainer().run(true, false, op);
			} catch (InvocationTargetException e) {
				WizardsPlugin.log(e);
			} catch (InterruptedException e) {
				WizardsPlugin.log(e);
			}

		}
		return true;
	}

	protected void addAcceleoNature(IProgressMonitor monitor) {
		// org.eclipse.jdt.core.javanature

		IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(getProjectName());
		try {
			IProjectDescription description = project.getDescription();
			addNature(description, "org.eclipse.acceleo.ide.ui.acceleoNature");
			addNature(description, "org.eclipse.jdt.core.javanature");

			// ICommand builderCommand =
			// String builderID = "org.eclipse.acceleo.ide.ui.acceleoBuilder";
			addBuilder(description, "org.eclipse.acceleo.ide.ui.acceleoBuilder");
			addBuilder(description, "org.eclipse.jdt.core.javabuilder");
			project.setDescription(description, monitor);
		} catch (CoreException e) {
			WizardsPlugin.log(e);
		}
	}

	/**
	 * @param description
	 * @param string
	 * @throws CoreException
	 */
	private void addNature(IProjectDescription description, String string) throws CoreException {
		if (!hasProjectNature(description, string)) {
			String[] natureIds = description.getNatureIds();
			String[] newNatureIds = new String[natureIds.length + 1];
			System.arraycopy(natureIds, 0, newNatureIds, 0, natureIds.length);
			newNatureIds[natureIds.length] = string;
			description.setNatureIds(newNatureIds);

		}
	}

	/**
	 * @param description
	 * @param builderID
	 * @return
	 * @throws CoreException
	 */
	private ICommand addBuilder(IProjectDescription description, String builderID) throws CoreException {
		ICommand projectBuilderCommand = getProjectBuilderCommand(description, builderID);
		if (projectBuilderCommand != null) {
			ICommand command = description.newCommand();
			command.setBuilderName(builderID);
			// Commit the spec change into the project
			addProjectBuilderCommand(description, command);
		}
		return projectBuilderCommand;
	}

	private void createProject(IProgressMonitor monitor) throws CoreException {
		if (monitor != null) {
			monitor.subTask("creating project " + ATL_PROJECT_NAME);
			if (monitor.isCanceled())
				return;
		}

		IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();

		project = workspaceRoot.getProject(ATL_PROJECT_NAME);
		if (!project.exists())
			project.create(monitor);

		project.open(monitor);
	}

	private boolean createFilesFromTemplate(String templatePath, Set<String> excludeFiles, String targetDirectoryName,
			IProgressMonitor monitor) throws MalformedURLException, CoreException {
		Bundle extensionBundle = WizardsPlugin.getDefault().getBundle();
		IContainer target = getTargetContainer();
		if (!"../".equals(targetDirectoryName)) {
			IFolder folder = target.getFolder(new Path(targetDirectoryName));
			if (!folder.exists())
				folder.create(false, true, null);
			target = folder;
		}
		Enumeration<?> entryPaths = extensionBundle.getEntryPaths(templatePath);

		String apath = buildAbsolutePath(target.getLocationURI().toURL().getPath());

		initVariables("", target.getFullPath().toString(), apath, getModelFile());
		processPath(entryPaths, target, extensionBundle, templatePath, excludeFiles, monitor);

		return true;
	}

	public void addPages() {
		projectPage = new AtlFlowProjectPage("Create a Atl flow example transformations");
		projectPage.setTitle("Template transformations");
		projectPage.setDescription("Some template transformations.");
		projectPage.setImageDescriptor(WizardsPlugin.imageDescriptorFromPlugin(WizardsPlugin.getId(),
				"icons/AtlflowDocument_template_newWizard_org.png"));

		addPage(projectPage);

	}
}
